
--TODO update assignment when changing dropdown

------------
--menu entry
------------
renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Ledger`s Scripts:Virtual MIDI Piano Helper Preferences...",  
  invoke =  function() preferences_gui()
  end 
}

--DELETE-------------------------------------------------
renoise.tool():add_keybinding {
  name = "Global:Tools:Virtual MIDI Piano Helper Preferences...",  
  invoke = function() preferences_gui()
  end  
}
------------------------------------------------
--TOOL XML PREFERENCES
------------------------------------------------
local options = renoise.Document.create {
  pref_midi_out = "",
  pref_midi_index = 1,
  enabled = false
}
--Assign options XML object to tool preferences 
renoise.tool().preferences = options
------------------------------------------------
--To get/set, append  options."xyz".value
------------------------------------------------
--I.e.  options.pref_midi_out.value
------------------------------------------------
------------------------------------------------

--the MIDI out that the current instrument is assigned to
local stored_instrument_midi_assignment = nil
--Dynamic midi assign tool, changes MIDI out to current instrument
last_instrument_index = nil
--dialog
local my_dialog = nil
--Viewbuilder object. Its methods are called to create GUI elements.


-----------------------------------------------
--Helper to convert dec to hex
-----------------------------------------------
local function num_to_hex_string(num)

 num = tostring(num)
 num = string.format("%X",num)
 
 if #num < 2 then
   return tostring("0x0"..num)
 end
 return tostring("0x"..num) --string
end
-----------------------------

--------------------------
--GUI
--------------------------
function preferences_gui()
  
  --check if dialog exists, if it does then close it (acts as toggle)
  if my_dialog and my_dialog.visible then
    my_dialog:close() 
    return
  end
  
  local vb = renoise.ViewBuilder()

  local rns = renoise.song()
  local selected_ins = renoise.song().selected_instrument_index
  --get available midi devices the user has in renoise
  local available_midi_outs = renoise.Midi.available_output_devices()
  
  -----------------
  --Build the GUI
  -----------------

  
  --define dialogs content
  local dialog_content =
    vb:vertical_aligner {
      margin = 8,
      mode = "center",
      --ENABLE Text
       vb:text{
         text = "Enable: ",
       },
       --ENABLE Checkbox
       vb:checkbox{
        value = options.enabled.value,
        id = "checkbox",
        notifier = function(value)
          
          --update preferences: tool,enabled or not
          options.enabled.value = value
          ---------------------------------------------
          --Send Notes-OFF messages to all notes
          --in case of stuck notes on the piano GUI
          ---------------------------------------------
          --Grab the chosen MIDI out
          local mdev = renoise.Midi.create_output_device(options.pref_midi_out.value)
                              
          --send note_off messages (0x80) to all notes (1-127)
          ---this requires helper function: num_to_hex_string(i) defined above---
          for i = 1,127 do
            local note = tonumber(num_to_hex_string(i))
            mdev:send({0x80,note})
          end
         
          --release the MIDI out
          mdev:close()
          
          ------------------------------------------------
          --Clear the MIDI assignments for all instruments
          ------------------------------------------------ 
          local rns = renoise.song()
          local num_instruments = #rns.instruments
          for i = 1,num_instruments do
            rns.instruments[i].midi_output_properties.device_name = ""
          end
          --clear stored MIDI assigment
          stored_instrument_midi_assignment = ""
          --update status
         -- vb.views["status"].text = "All Instrument Midi Assignments Set To `None`"
         ------------------------------------------------------------------------------
 
          --update status
          if value == true then
            vb.views["status"].text = "Tool Enabled"
            --update current instrument
            renoise.song().instruments[renoise.song().selected_instrument_index].midi_output_properties.device_name = options.pref_midi_out.value
          else
            vb.views["status"].text = "Tool Disabled"
             --update current instrument
             renoise.song().instruments[renoise.song().selected_instrument_index].midi_output_properties.device_name = (stored_instrument_midi_assignment or "")
          end
        end
       },
      --CHOOSER TEXT
       vb:text{
         text = "Choose MIDI Out To Route To: ",
       },
      --MIDI OUT CHOOSER
       vb:popup {
         items = available_midi_outs,
         id = "midi device",
         width = 200,
         value = options.pref_midi_index.value,
         notifier = function(value)
           --set preferences so will be kept for next session
           options.pref_midi_out.value = available_midi_outs[value]
           --store the value for next session
           options.pref_midi_index.value = value
          
          --if the tool is enabled then update the current instrument asiignment
          if options.enabled.value == true then
            --update current instrument
            renoise.song().instruments[renoise.song().selected_instrument_index].midi_output_properties.device_name = (options.pref_midi_out.value or "")
            --clear stored MIDI assigment
            stored_instrument_midi_assignment = ""
            --update status
            vb.views["status"].text = "Current Assignments set to "..(options.pref_midi_out.value or "")
          end
        end,
       },
       --[[ --REFRESH CURRENT "" TEXT 
       vb:text{
         text = "Refresh Current Instrument To Chosen Output:",
       },
       vb:button{
        text = "Refresh Current",
        notifier = function()
          --update current instrument
          renoise.song().instruments[renoise.song().selected_instrument_index].midi_output_properties.device_name = (options.pref_midi_out.value or "")
          --clear stored MIDI assigment
          stored_instrument_midi_assignment = ""
          --update status
          vb.views["status"].text = "Current Assignments set to "..(options.pref_midi_out.value or "")
        end
       },--]]
       --RESET ALL OUTS TO "" TEXT 
      --[[ vb:text{
         text = "Set All Instrument Outputs To `None`: ",
       },
       vb:button{
        text = "Clear All Instrument MIDI Assignments",  
        notifier = function()
          local rns = renoise.song()
          local num_instruments = #rns.instruments
          for i = 1,num_instruments do
            rns.instruments[i].midi_output_properties.device_name = ""
          end
          --clear stored MIDI assigment
          stored_instrument_midi_assignment = ""
          --update status
          vb.views["status"].text = "All Instrument Midi Assignments Set To `None`"
        end
       },--]]
      
       --STATUS
       vb:text{
        text = "Status: ",
        font = "bold",
       },  
       
       --STATUS
       vb:text{
        text = "... ",
        id = "status",
       },        
      }
      
    --key Handler
    local function my_keyhandler_func(dialog,key)
    
       --always focus the pattern editor so renoise responds to key input
      -- renoise.app().window.lock_keyboard_focus = false
      -- renoise.app().window.lock_keyboard_focus = true
    
       --if escape pressed then close the dialog else return key to renoise
       if not (key.modifiers == "" and key.name == "esc") then
          return key
       else
         dialog:close()
       end
    end     
      

     --Show the script dialog
     my_dialog = renoise.app():show_custom_dialog(
       "Virtual MIDI Piano Helper Preferences", dialog_content,my_keyhandler_func)--
 
end


----------------------------------------------------------------------
----------------------------------------------------------------------
----------------------------------------------------------------------

---main called each time instrument index changes
function main()
  
  local rns = renoise.song()
  
  --if tool not enabled then return early
  if options.enabled.value == false then
    return
  end
  
  local rns = renoise.song()  
  --get the current instrument index
  local current_instrument = rns.selected_instrument_index

  --reset previous instruments midi assignment to what it was before the tool changed it
  if last_instrument_index then
    rns.instruments[last_instrument_index].midi_output_properties.device_name = stored_instrument_midi_assignment
  end
  
  --set previous instrument index to current for next time
  last_instrument_index = current_instrument

  --get current instrument MIDI assignment (this is done before tool changes it)
  stored_instrument_midi_assignment = rns.instruments[current_instrument].midi_output_properties.device_name
  
  
  --CAN THIS BE PUT AT START OF FUNCTION AND RETURNED EARLY??
  --Set current instruments to user chosen MIDI Out
  if rns.instruments[current_instrument].name ~= "" then
   rns.instruments[current_instrument].midi_output_properties.device_name = options.pref_midi_out.value
  end
  
  --print all instrument outs
 --[[ local num_instruments = #rns.instruments
    for i = 1,num_instruments do
      print( rns.instruments[i].midi_output_properties.device_name)
    end--]]

end --end of main()


--------------------------------
--Notifiers
--------------------------------

------------------------------
--add notifier on new document
------------------------------
renoise.tool().app_new_document_observable:add_notifier (function() 
 
  local rns = renoise.song()
      
  --check if the dialog exists, if it does then close it (acts as toggle)
  if my_dialog and my_dialog.visible then
    my_dialog:close() 
  end
  
  --add notifier to each new instrument found
  -------------------------------------------
  local num_instruments = #renoise.song().instruments
  for i = 1,num_instruments do
    renoise.song().instruments[i].name_observable:add_notifier(function() main() end)
  end
  
  --add notifier for instrument index change
  ------------------------------------------
  renoise.song().selected_instrument_index_observable:add_notifier(function() main() end)
  
  
  --reset last index to nil in case instrument slot does not exist in new song
  last_instrument_index = nil
   
  --clear all MIDI assignments in the new document if tool enabled
  if options.enabled.value == true then
    local num_instruments = #rns.instruments
    for i = 1,num_instruments do
      rns.instruments[i].midi_output_properties.device_name = ""
    end
  end
 
  --current instrument index
  local current_instrument = renoise.song().selected_instrument_index
  --get midi assignment to the current instrument MIDI assignment
  stored_instrument_midi_assignment = renoise.song().instruments[current_instrument].midi_output_properties.device_name
  
  --run main()
  main()
    
 end
)

----------------------------------------------------------------------
--notifiers
----------------------------------------------------------------------

--add notifier to each instrument slot
--[[function add_main_to_inst_name_notifiers()
  local num_instruments = #renoise.song().instruments
  for i = 1,num_instruments do
    renoise.song().instruments[i].name_observable:add_notifier(function() main() end)
  end
end--]]

-------------------------------
--on selected instrument change
-------------------------------
--[[function add_main_to_inst_index_notifier()  
  renoise.song().selected_instrument_index_observable:add_notifier(function()
     main() end)
end--]]




